%--------------------------------------------------------------------
%LPV_Synthesis (Numerical Example Demonstration)
%Update:03/13/2014
%Shu Wang
%--------------------------------------------------------------------
clear all;
clc;
%% Model Construction

rho=[2:0.5:7];  % time varying parameter rho
% rho = 3.5;
Np=size(rho,2);

tau=sqrt(133.6-16.8.*rho);  % system parameter tau
K=sqrt(4.8.*rho-8.6);   % system parameter K
 
Ag=-1./tau; % SS model matrices for the plant
Bg=1./tau;
Cg=K;
Dg=0;

G = ss;
for i=1:Np
    Agi=Ag(i);
    Bgi=Bg(i);
    Cgi=Cg(i);
    G(:,:,i) = ss(Agi,Bgi,Cgi,Dg);
end

%%
% Weighted Interconnection
Wu = makeweight(0.1,10,10);
We = makeweight(10,10,0.1);
Wd = 0.1;

systemnames = 'G Wu We Wd';
inputvar = '[d; r; u]';
outputvar = '[ Wu; We; r-G]';
input_to_We = '[r-G]';
input_to_Wu = '[u]';
input_to_Wd = '[d]';
input_to_G = '[Wd+u]';
cleanupsysic = 'yes';
Pic = sysic;

%%
nmeas = 1;
ncon = 1;
maxiter = 20;

Psi11 = 1;
Psi22 = 1;

gamold = inf;
gamtol = 0.05;

%%
% Perform true LPV design with gridded approach
for ii = 1:maxiter
%     ii = 2;
    fprintf('\n Iteration: %i \n', ii);
    
    scaleinput = blkdiag(inv(Psi22),1,1);
    scaleoutput = blkdiag(Psi11,1,1);
    Picscaled = scaleoutput*Pic*scaleinput;
    %%
    % Expected Result: GAMGrid = 3.1034 (With Default "BackOff" method)
    % This "backs off" the optimal gamma by BackOffFactor = 1.2000
    opt = lpvsynOptions;
    opt.BackOffFactor = 1.1;

    [KGrid,GAMGrid,INFOGrid] = lpvsyneng(Picscaled,nmeas,ncon);
%     [KGrid,CL,GAMGrid,INFOGrid] = hinfsyn(Picscaled,nmeas,ncon);
    fprintf('Gamma controller step: %f \t \n', GAMGrid)
    
    %% robust performance analysis step
    %get analysis interconnection
    Nic = lft(Pic,KGrid);
    
    
    % Specify IQCs
    psi{1} = ss(eye(2));
    D2 = 1e-2*ss(tf([1 1e2],[1 1]));
    psi{2} = blkdiag(D2,D2);
    
    % options for bisection algorithm
    gamlb = 0; %lower bound
    gamub = GAMGrid; %upper bound
    tol = 1e-3; %tolerance for bisection
    
    while gamub-gamlb>tol
        gamtry = (gamlb+gamub)/2;
        
        %run analysis
        [t,info] = simplerobperf(Nic,[1,1],psi,gamtry);
        
        %new bounds for bisection
        if t <= 0
            gamub = gamtry;
        else
            gamlb = gamtry;
        end
    end
    
    gamopt = gamub;
    %run analysis
    [t,info] = simplerobperf(Nic,[1,1],psi,gamopt);

    fprintf('Gamma analysis step: %f \t \n', gamopt)
    
    %%
    lam = info.lam
    M = blkdiag(1,-gamopt^2);
    Pi = ss(zeros(2,2));
    for jj = 1:length(lam)
        Pi = Pi + lam(jj)*psi{jj}'*M*psi{jj};
    end
    
    [Psi,M] = jspec(Pi);

    Psi = blkdiag(gamopt,1)*Psi;

    Psi11 = Psi(1,1);
    Psi22 = Psi(2,2);
    
    gamvec(ii) = gamopt;
    
    if gamold-gamopt < gamtol
        break;
    end
    gamold = gamopt;
    
    
end

%%

% plot(1:9, gamvec1,'x-','color',mycolor('maroon'),'MarkerSize',12)
% hold on;
% plot(1:10, gamvec,'o--','color',mycolor('coolblue'),'MarkerSize',8)
% xlabel('Iteration [-]')
% ylabel('Robust Perforamnce [-]')
% garyfyFigure;
% box on
% legend('\Psi_1 = I and \Psi_2 = ','\Psi = I','Location','NorthEast')
% matlab2tikz('lambdakiteration.tex')
 
